////////////////////////////////////////////////////////////
//Test SL018
//ARM C source code
//LM3S600(6MHz) + EWARM 5.30
////////////////////////////////////////////////////////////   
#include "hw_types.h"
#include "hw_memmap.h"
#include "hw_ints.h"
#include "string.h"
#include "sysctl.h"
#include "gpio.h"
#include "systick.h"
#include "interrupt.h"
#include "i2c.h"

//============================================
//  Command List, preamble + length + command 
//============================================
const unsigned char SL018CMD_ComSelectCard[]          ={1,1};
const unsigned char SL018CMD_ComLoginSector0[]        ={9,2,0+2,0xAA,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
const unsigned char SL018CMD_ComReadBlock1[]          ={2,3,1+8};
const unsigned char SL018CMD_ComWriteBlock1[]         ={18,4,1+8,0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,0xAA,0xBB,0xCC,0xDD,0xEE,0xFF};
const unsigned char SL018CMD_ComIntiPurse1[]          ={6,6,1+8,0x78,0x56,0x34,0x12};
const unsigned char SL018CMD_ComReadPurse1[]          ={2,5,1+8};       
const unsigned char SL018CMD_ComIncrPurse1[]          ={6,8,1+8,0x02,0x00,0x00,0x00};                                
const unsigned char SL018CMD_ComDecrPurse1[]          ={6,9,1+8,0x01,0x00,0x00,0x00};                                
const unsigned char SL018CMD_ComCopyValue[]           ={3,0x0A,1+8,2+8};
const unsigned char SL018CMD_ComReadUltralightPage5[] ={2,0x10,0x05};
const unsigned char SL018CMD_ComWriteUltralightPage5[]={6,0x11,0x05,0x12,0x34,0x56,0x78};
const unsigned char SL018CMD_ComRedLedOn[]            ={2,0x40,1};
const unsigned char SL018CMD_ComRedLedOff[]           ={2,0x40,0};

//============================================
//  global variable define
//============================================
unsigned char g_cCardType;
unsigned long int g_iTimer;
unsigned char g_bOverTime;
unsigned char g_cI2CRxBuf[60];

//============================================
//  i/o port define
//============================================
#define CARDIN_SL018 GPIO_PORTB_BASE,GPIO_PIN_1

//============================================
//  i2c slave address define
//============================================
#define SLVADD 0x50

//============================================
//  procedure define
//============================================
void InitializeSystem(); 
void Start_Time(unsigned long int ms);
void Stop_Time();
void Test_SL018();

void main()
{ InitializeSystem();
  g_iTimer=0;
  while(1) Test_SL018();
}

//============================================
//	system clock interrupt procedure
//============================================
void SysTickIntHandler(void)
{ if(g_iTimer!=0)
  { g_iTimer--;
    if(g_iTimer==0) g_bOverTime=1;
  }
}

//============================================
//  initialize system hardware config
//  system clock,i2c interface
//============================================
void InitializeSystem()
{	
  //init sysclock
  //period=1ms
  SysCtlClockSet(SYSCTL_SYSDIV_4|SYSCTL_USE_PLL|SYSCTL_OSC_MAIN|SYSCTL_XTAL_6MHZ);
  SysTickPeriodSet(SysCtlClockGet()/1000);
  SysTickEnable();
  SysTickIntEnable();
  
  //init io port
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
  GPIODirModeSet(GPIO_PORTB_BASE,GPIO_PIN_1,GPIO_DIR_MODE_IN);
  GPIOPadConfigSet(GPIO_PORTB_BASE,GPIO_PIN_1,GPIO_STRENGTH_2MA,GPIO_PIN_TYPE_STD_WPU);
  GPIODirModeSet(GPIO_PORTE_BASE,GPIO_PIN_0,GPIO_DIR_MODE_OUT);
  GPIOPadConfigSet(GPIO_PORTE_BASE,GPIO_PIN_0,GPIO_STRENGTH_8MA,GPIO_PIN_TYPE_STD);
  
  //init i2c interface
  //SCL=PB2,SDA=PB3
  GPIOPinTypeI2C(GPIO_PORTB_BASE,GPIO_PIN_2|GPIO_PIN_3);
  SysCtlPeripheralEnable(SYSCTL_PERIPH_I2C);
  I2CMasterInitExpClk(I2C_MASTER_BASE,SysCtlClockGet(),true);
  
  //interrupt enable
  IntMasterEnable();
}

//============================================
//  start timer
//============================================
void Start_Time(unsigned long int ms)
{ g_iTimer=ms;
  g_bOverTime=0;
}

//============================================
//  stop timer
//============================================
void Stop_Time()
{ g_iTimer=0;
}

//============================================
//  send buf by i2c hardware interface
//============================================
void SendBuf_I2C(unsigned char *dat,unsigned char len)
{ unsigned char counter;
  I2CMasterSlaveAddrSet(I2C_MASTER_BASE,SLVADD,false);
  I2CMasterDataPut(I2C_MASTER_BASE,dat[0]);
  I2CMasterControl(I2C_MASTER_BASE,I2C_MASTER_CMD_BURST_SEND_START);
  while(I2CMasterBusy(I2C_MASTER_BASE));
  for(counter=1;counter<len-1;counter++)
  { I2CMasterDataPut(I2C_MASTER_BASE,dat[counter]);
    I2CMasterControl(I2C_MASTER_BASE,I2C_MASTER_CMD_BURST_SEND_CONT);
    while(I2CMasterBusy(I2C_MASTER_BASE));
  }
  I2CMasterDataPut(I2C_MASTER_BASE,dat[counter]);
  I2CMasterControl(I2C_MASTER_BASE,I2C_MASTER_CMD_BURST_SEND_FINISH);
  while(I2CMasterBusy(I2C_MASTER_BASE));
}

//============================================
//  receive buf by i2c hardware interface
//============================================
unsigned char ReadBuf_I2C(unsigned char *dst,unsigned char BufSize)
{ 
  unsigned char counter;
  unsigned long int cnt;
  
  I2CMasterSlaveAddrSet(I2C_MASTER_BASE,SLVADD,true);
  I2CMasterControl(I2C_MASTER_BASE,I2C_MASTER_CMD_BURST_RECEIVE_START);
  cnt=10000;
  while((I2CMasterBusy(I2C_MASTER_BASE))&&(--cnt));
  if(cnt==0) return 0;
  dst[0]=I2CMasterDataGet(I2C_MASTER_BASE);
  if(dst[0]>=BufSize) return 0;
  for(counter=1;counter<dst[0];counter++)
  { I2CMasterControl(I2C_MASTER_BASE,I2C_MASTER_CMD_BURST_RECEIVE_CONT);
    cnt=10000;
    while((I2CMasterBusy(I2C_MASTER_BASE))&&(--cnt));
    dst[counter]=I2CMasterDataGet(I2C_MASTER_BASE);
  }
  I2CMasterControl(I2C_MASTER_BASE,I2C_MASTER_CMD_BURST_RECEIVE_FINISH);
  cnt=10000;
  while((I2CMasterBusy(I2C_MASTER_BASE))&&(--cnt));
  if(cnt==0) return 0;
  dst[counter]=I2CMasterDataGet(I2C_MASTER_BASE);
  return 1;
}

void Test_SL018()
{
  #define RCVCMD_SL018 g_cI2CRxBuf[1]
  #define RCVSTA_SL018 g_cI2CRxBuf[2]
  #define DELAYTIME 50
  unsigned long int lPurseValue;
  g_cCardType=0xff;	
  
  if((GPIOPinRead(CARDIN_SL018)&GPIO_PIN_1)==GPIO_PIN_1) return;
  
  SendBuf_I2C((unsigned char *)(SL018CMD_ComSelectCard),sizeof(SL018CMD_ComSelectCard));
  Start_Time(DELAYTIME);
  while(g_bOverTime==0);
  Stop_Time();
  if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
  if((RCVCMD_SL018!=0x01)||(RCVSTA_SL018!=0)) return;
  g_cCardType=g_cI2CRxBuf[g_cI2CRxBuf[0]];
  
  switch(g_cCardType)
  {
    case 1:    //Mifare 1k 4 BYTE UID
    case 2:    //Mifare 1k 7 BYTE UID
    case 4:    //Mifare 4k 4 BYTE UID
    case 5:    //Mifare 4k 7 BYTE UID 
      SendBuf_I2C((unsigned char *)(SL018CMD_ComLoginSector0),sizeof(SL018CMD_ComLoginSector0));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x02)||(RCVSTA_SL018!=0x02)) return;
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComWriteBlock1),sizeof(SL018CMD_ComWriteBlock1));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x04)||(RCVSTA_SL018!=0x00)) return;
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComReadBlock1),sizeof(SL018CMD_ComReadBlock1));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x03)||(RCVSTA_SL018!=0x00)) return;
      if(memcmp(&SL018CMD_ComWriteBlock1[3],&g_cI2CRxBuf[3],16)!=0) return;
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComIntiPurse1),sizeof(SL018CMD_ComIntiPurse1));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x06)||(RCVSTA_SL018!=0x00)) return;
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComIncrPurse1),sizeof(SL018CMD_ComIncrPurse1));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x08)||(RCVSTA_SL018!=0x00)) return;
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComDecrPurse1),sizeof(SL018CMD_ComDecrPurse1));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x09)||(RCVSTA_SL018!=0x00)) return;
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComReadPurse1),sizeof(SL018CMD_ComReadPurse1));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x05)||(RCVSTA_SL018!=0x00)) return;
      //Check value	 
      lPurseValue=g_cI2CRxBuf[6];
      lPurseValue=(lPurseValue<<8)+g_cI2CRxBuf[5];
      lPurseValue=(lPurseValue<<8)+g_cI2CRxBuf[4];
      lPurseValue=(lPurseValue<<8)+g_cI2CRxBuf[3];
      if(lPurseValue!=0x12345678+0x00000002-0x00000001) return;
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComCopyValue),sizeof(SL018CMD_ComCopyValue));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x0a)||(RCVSTA_SL018!=0x00)) return;
      //Glare Red_Led to indicate working ok
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComRedLedOn),sizeof(SL018CMD_ComRedLedOn));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x40)||(RCVSTA_SL018!=0x00)) return;
      Start_Time(300);
      while(g_bOverTime==0);
      Stop_Time();
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComRedLedOff),sizeof(SL018CMD_ComRedLedOff));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x40)||(RCVSTA_SL018!=0x00)) return;
      break;
    
 
    case 3:    //Mifare Ultralight
      SendBuf_I2C((unsigned char *)(SL018CMD_ComWriteUltralightPage5),sizeof(SL018CMD_ComWriteUltralightPage5));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x11)||(RCVSTA_SL018!=0x00)) return;
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComReadUltralightPage5),sizeof(SL018CMD_ComReadUltralightPage5));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x10)||(RCVSTA_SL018!=0x00)) return;
      if(memcmp(&SL018CMD_ComWriteUltralightPage5[3],&g_cI2CRxBuf[3],4)!=0) return;
      
      //Glare Red_Led to indicate working ok
      SendBuf_I2C((unsigned char *)(SL018CMD_ComRedLedOn),sizeof(SL018CMD_ComRedLedOn));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x40)||(RCVSTA_SL018!=0x00)) return;
      Start_Time(300);
      while(g_bOverTime==0);
      Stop_Time();
      
      SendBuf_I2C((unsigned char *)(SL018CMD_ComRedLedOff),sizeof(SL018CMD_ComRedLedOff));
      Start_Time(DELAYTIME);
      while(g_bOverTime==0);
      Stop_Time();
      if(ReadBuf_I2C(g_cI2CRxBuf,sizeof(g_cI2CRxBuf))==0) return;
      if((RCVCMD_SL018!=0x40)||(RCVSTA_SL018!=0x00)) return;
      break;
    

    default:
      break;
  }
}

